from django_filters.rest_framework import DjangoFilterBackend

from smoothglue.core.views import ListModelViewSet
from smoothglue.map.models import ImageryLayer
from smoothglue.map.serializers.imagery_layer import ImageryLayerSerializer


class ImageryLayerViewset(ListModelViewSet):
    filter_backends = [DjangoFilterBackend]
    queryset = ImageryLayer.objects.filter(is_active=True).order_by("sort_order")
    serializer_class = ImageryLayerSerializer



import re


def get_decimal_degrees_from_dms_string(dms_string):
    """
    Convert set data representing lat lon as a Degree Minute Second (DMS) value to Decimal Degrees
    using formula: Decimal Degrees = degrees + (minutes/60) + (seconds/3600)

    ACO coordinate set data comes in 2 varieties:
    LATS (e.g. LATS:431026N1165028W) and LATM (e.g. LATM:4302N11553W)
    Assumptions:
        LATS format is Degrees Minutes Seconds and broken down as follows:
        Index: 0  1  2  3  4  5  6  7
        LATS:  43 10 26 N 116 50 28 W

    """
    index_dict = {
        "lat_degree_index": 0,
        "lat_minute_index": 1,
        "lat_second_index": 2,
        "lat_millisecond_index": 3,
        "lat_direction_index": 4,
        "lon_degree_index": 5,
        "lon_minute_index": 6,
        "lon_second_index": 7,
        "lon_millisecond_index": 8,
        "lon_direction_index": 9,
    }

    pattern = r"\s*(\d{2})+(\d{2})+(\d{2})+(\.\d{2})+([N|S])*(\d{3})+(\d{2})+(\d{2})+(\.\d{2})+([E|W])"

    try:
        lat_lon_set = re.findall(pattern, dms_string)[0]
    except IndexError as exc:
        raise ValueError(
            f"There was an error parsing coordinate: {dms_string}."
        ) from exc

    lat_decimal = (
        int(lat_lon_set[index_dict["lat_degree_index"]])
        + int(lat_lon_set[index_dict["lat_minute_index"]]) / 60
    )
    lon_decimal = (
        int(lat_lon_set[index_dict["lon_degree_index"]])
        + int(lat_lon_set[index_dict["lon_minute_index"]]) / 60
    )

    lat_decimal += (
        float(
            lat_lon_set[index_dict["lat_second_index"]]
            + lat_lon_set[index_dict["lat_millisecond_index"]]
        )
        / 3600
    )
    lon_decimal += (
        float(
            lat_lon_set[index_dict["lon_second_index"]]
            + lat_lon_set[index_dict["lon_millisecond_index"]]
        )
        / 3600
    )

    if (
        lat_lon_set[index_dict["lon_direction_index"]] == "S"
        or lat_lon_set[index_dict["lon_direction_index"]] == "W"
    ):
        lon_decimal *= -1

    # GeoJSON coordinates are in (lon, lat) format
    return [lon_decimal, lat_decimal]



from django.contrib import admin

from smoothglue.core.admin import DataAdmin
from smoothglue.map.models import ImageryLayer


class ImageryLayerAdmin(DataAdmin):
    list_display = (
        "name",
        "provider_type",
        "is_active",
    )
    list_editable = ("is_active",)
    list_filter = ("provider_type",)


admin.site.register(ImageryLayer, ImageryLayerAdmin)



# Generated by Django 4.2.13 on 2024-06-18 00:51

import uuid

from django.db import migrations, models


class Migration(migrations.Migration):
    initial = True

    dependencies = []

    operations = [
        migrations.CreateModel(
            name="ImageryLayer",
            fields=[
                (
                    "id",
                    models.UUIDField(
                        default=uuid.uuid4,
                        editable=False,
                        primary_key=True,
                        serialize=False,
                        unique=True,
                    ),
                ),
                ("name", models.CharField(blank=True, max_length=64, unique=True)),
                ("url", models.CharField(blank=True, max_length=512)),
                (
                    "provider_type",
                    models.CharField(
                        choices=[
                            ("NO", "None"),
                            ("AG", "ArcGis Map Server"),
                            ("TL", "Tile Map Service"),
                            ("BG", "Bing Map Imagery"),
                            ("GG", "Google Earth Enterprise"),
                            ("OS", "Open Street Map"),
                            ("WM", "Web Map Tile Service"),
                            ("UT", "Url Template Imagery"),
                            ("GO", "Geo Server"),
                        ],
                        default="NO",
                        max_length=2,
                    ),
                ),
                ("sort_order", models.IntegerField(blank=True, null=True)),
                ("is_active", models.BooleanField(default=True)),
            ],
        ),
    ]



import uuid

from django.db import models


class ImageryLayer(models.Model):
    class MapImageryProviderType(models.TextChoices):
        NONE = "NO", "None"
        ARCGIS = "AG", "ArcGis Map Server"
        TILE = "TL", "Tile Map Service"
        BING = "BG", "Bing Map Imagery"
        GOOGLE = "GG", "Google Earth Enterprise"
        OPEN = "OS", "Open Street Map"
        WEBMAP = "WM", "Web Map Tile Service"
        URL = "UT", "Url Template Imagery"
        GEO = "GO", "Geo Server"

    id = models.UUIDField(
        primary_key=True, unique=True, default=uuid.uuid4, editable=False
    )
    name = models.CharField(max_length=64, unique=True, blank=True, null=False)
    url = models.CharField(max_length=512, blank=True, null=False)
    provider_type = models.CharField(
        max_length=2,
        choices=MapImageryProviderType.choices,
        default=MapImageryProviderType.NONE,
    )
    sort_order = models.IntegerField(blank=True, null=True)
    is_active = models.BooleanField(default=True)

    def __str__(self) -> str:
        return f"{self.name} - {self.provider_type}"



from rest_framework import serializers

from smoothglue.core.serializers.dynamic import DynamicModelSerializer
from smoothglue.map.models import ImageryLayer


class ImageryLayerSerializer(DynamicModelSerializer):

    provider_type = serializers.CharField(
        source="get_provider_type_display", read_only=True
    )

    class Meta:
        model = ImageryLayer
        fields = ["id", "name", "url", "provider_type", "sort_order", "is_active"]



# boot_django.py
#
# This file sets up and configures Django. It's used by scripts that need to
# execute as if running in a Django server.
import os

import django
from django.conf import settings

BASE_DIR = os.path.abspath(os.path.join(os.path.dirname(__file__), "smoothglue/map"))


def boot_django():
    # The settings.configure() call takes a list of args that are equivalent to variables
    # defined in a settings.py file.
    # Anything you would need in your settings.py to make your app run gets passed here
    settings.configure(
        BASE_DIR=BASE_DIR,
        DEBUG=True,
        DATABASES={
            "default": {
                "ENGINE": "django.db.backends.sqlite3",
                "NAME": os.path.join(BASE_DIR, "db.sqlite3"),
            }
        },
        INSTALLED_APPS=[
            "django.contrib.contenttypes",
            "django.contrib.auth",
            "smoothglue.map",
        ],
        TIME_ZONE="UTC",
        USE_TZ=True,
    )
    django.setup()



from django.urls import include, path
from rest_framework import routers

from smoothglue.map import views

ROUTER = routers.DefaultRouter()

ROUTER.register(
    r"imagelayersviews", views.ImageryLayerViewset, basename="imagelayersviews"
)


urlpatterns = [path("", include(ROUTER.urls))]



from django.test import TestCase

from smoothglue.map.utils.coordinate_utils import get_decimal_degrees_from_dms_string


class TestCoordinateUtilsSerializer(TestCase):

    def test_serializer_valid(self):

        coordinate_str = "373420.50N1272218.92E"
        result = get_decimal_degrees_from_dms_string(coordinate_str)
        self.assertEqual(result, [127.37192222222221, 37.572361111111114])



from django.test import TestCase

from smoothglue.core.tests.utils import CommonSerializer
from smoothglue.map.models import ImageryLayer
from smoothglue.map.serializers.imagery_layer import ImageryLayerSerializer


class TestImageryLayerJsonSerializer(TestCase):
    def setUp(self):
        self.image_layer_data = {
            "name": "test",
            "url": "http://braingu.com",
            "provider_type": "ArcGisMapServerImageryProvider",
        }

        self.required_fields = []

    def test_serializer_valid(self):
        CommonSerializer(serializer=ImageryLayerSerializer).test_model_serializer_valid(
            model=ImageryLayer,
            required_fields=self.required_fields,
            data=self.image_layer_data,
        )
